@@ -1,7 +1,7 @@
 /*
  * @(#)AbacusD.c
  *
- * Copyright 1999 - 2014  David A. Bagley, bagleyd AT verizon.net
+ * Copyright 1999 - 2018  David A. Bagley, bagleyd AT verizon.net
  *
  * Abacus demo and neat pointers from
  * Copyright 1991 - 1998  Luis Fernandes, elf AT ee.ryerson.ca
@@ -27,9 +27,9 @@
 #include "AbacusP.h"
 
 #ifndef WINVER
+static int displayedError = 0;
 static void initializeAbacusDemo(Widget request, Widget renew);
 static void exposeAbacusDemo(Widget renew, XEvent *event, Region region);
-
 #if 0
 static void resizeAbacusDemo(AbacusWidget w);
 
@@ -266,15 +266,27 @@ loadFont(AbacusWidget w)
 	/* Access font */
 	if (w->abacusDemo.font && (w->abacusDemo.fontInfo =
 			XLoadQueryFont(display, w->abacusDemo.font)) == NULL) {
-		(void) sprintf(buf,
-			"Can not open %s font.\nAttempting %s font as alternate\n",
+#ifdef HAVE_SNPRINTF
+		(void) snprintf(buf, 512,
+			"Cannot open %s font.\nAttempting %s font as alternate\n",
 			w->abacusDemo.font, altfontname);
+#else
+		(void) sprintf(buf,
+			"Cannot open %s font.\nAttempting %s font as alternate\n",
+			w->abacusDemo.font, altfontname);
+#endif
 		DISPLAY_WARNING(buf);
 		if ((w->abacusDemo.fontInfo = XLoadQueryFont(display,
 				altfontname)) == NULL) {
-			(void) sprintf(buf,
-				"Can not open %s alternate font.\nUse the -demofont option to specify a font to use.\n",
+#ifdef HAVE_SNPRINTF
+			(void) snprintf(buf, 512,
+				"Cannot open %s alternate font.\nUse the -demofont option to specify a font to use.\n",
 				altfontname);
+#else
+			(void) sprintf(buf,
+				"Cannot open %s alternate font.\nUse the -demofont option to specify a font to use.\n",
+				altfontname);
+#endif
 			DISPLAY_ERROR(buf);
 		}
 	}
@@ -287,7 +299,7 @@ loadFont(AbacusWidget w)
 #define setAbacusDemo(w,r) setAbacus(w,r)
 
 typedef enum {
-	INTRO, CHAPTER, QUERY, DISPLAY, CONCL
+	INTRO, BOOK, CHAPTER, QUERY, DISPLAY, CONCL
 } TextType;
 
 static const char *introFramelessText[LINES] =
@@ -306,56 +318,20 @@ static const char *introFramedText[LINES] =
 	"Type `o' to toggle demo or `q' to quit.",
 };
 
-static const char *chapterText[][LINES] =
+static const char *bookText[LINES] =
 {
-	{
-		"     1. Increment Chapter",
-		"Type: `r' to Read this chapter",
-		"      `n' to Next chapter",
-		"      `q' to Quit the demo",
-	},
-	{
-		"     2. Decrement Chapter",
-		"Type: `r' to Read this chapter",
-		"      `n' to Next chapter",
-		"      `q' to Quit the demo",
-	},
-	{
-		"     3. Addition Chapter",
-		"Type: `r' to Read this chapter",
-		"      `n' to Next chapter",
-		"      `q' to Quit the demo",
-	},
-	{
-		"     4. Subtraction Chapter",
-		"Type: `r' to Read this chapter",
-		"      `n' to Next chapter",
-		"      `q' to Quit the demo",
-	},
-	{
-		"     5. Multiplication Chapter",
-		"Type: `r' to Read this chapter",
-		"      `n' to Next chapter",
-		"      `q' to Quit the demo",
-	},
-	{
-		"     6. Division Chapter",
-		"Type: `r' to Read this chapter",
-		"      `n' to Next chapter",
-		"      `q' to Quit the demo",
-	},
-	{
-		"     7. Square Root Chapter",
-		"Type: `r' to Read this chapter",
-		"      `n' to Next chapter",
-		"      `q' to Quit the demo",
-	},
-	{
-		"     8. Cube Root Chapter",
-		"Type: `r' to Read this chapter",
-		"      `n' to Next chapter",
-		"      `q' to Quit the demo",
-	},
+	"  ",
+	"Type: `r' to Read this book",
+	"      `n' to Next book",
+	"      `q' to Quit the demo",
+};
+
+static const char *chapterText[LINES] =
+{
+	"     ",
+	"Type: `r' to Read this chapter",
+	"      `n' to Next chapter",
+	"      `q' to Quit the demo",
 };
 
 static const char *queryText[LINES] =
@@ -379,75 +355,113 @@ static const char *conclLesson[LINES] =
  * array */
 static char displayText[LINES][CHARS] = {"", "", "", ""};
 
-static char *
+static Boolean fallback = True;
+
+#if 0
+static const char *
 postfixType(int mode)
 {
 	switch (mode) {
 	case CHINESE:
-		return (char *) ""; /* cn */
+		return ""; /* cn */
 	case JAPANESE:
-		return (char *) "jp";
+		return "jp";
 	case KOREAN:
-		return (char *) "ko";
+		return "ko";
 	case ROMAN:
-		return (char *) "jp"; /* ro */
+		return "jp"; /* ro */
 	case RUSSIAN:
-		return (char *) "ru";
+		return "ru";
 	case DANISH:
-		return (char *) "ru"; /* dk */
+		return "ru"; /* dk */
 	case MEDIEVAL:
-		return (char *) ""; /* cn, me, ge, uk */
+		return ""; /* cn, me, ge, uk */
 	default:
-		return (char *) "";
+		return "";
+	}
+}
+#endif
+
+static unsigned int
+commonEdition(unsigned int mode)
+{
+	unsigned int edition;
+
+	switch (mode) {
+	case ROMAN:
+		edition = JAPANESE;
+		break;
+	case DANISH:
+		edition = RUSSIAN;
+		break;
+	case MEDIEVAL:
+		edition = CHINESE;
+		break;
+	default:
+		edition = mode;
+	}
+	return edition;
+}
+
+struct libraryType abacusLibrary;
+
+static unsigned int getNumberBooks(void) {
+	if (fallback)
+		return getFallbackBooks();
+	return abacusLibrary.books;
+}
+
+static unsigned int getNumberEditions(AbacusWidget w) {
+	if (fallback)
+		return getFallbackEditions();
+	if (abacusLibrary.book == NULL)
+		return 0;
+	return abacusLibrary.book[w->abacusDemo.bookCount].editions;
+}
+
+static unsigned int getEdition(AbacusWidget w) {
+	unsigned int edition = commonEdition((unsigned int) w->abacus.mode);
+	unsigned int editions = getNumberEditions(w);
+
+	if (edition >= editions) {
+#ifdef DEBUG
+		(void) fprintf(stderr, " getEdition forcing : %d\n", CHINESE);
+#endif
+		return CHINESE;
+	}
+#ifdef DEBUG
+	(void) fprintf(stderr, " getEdition returning : %d\n", edition);
+#endif
+	return edition;
+}
+
+static unsigned int getNumberChapters(AbacusWidget w) {
+	if (fallback)
+		return getFallbackChapters();
+	if (abacusLibrary.book[w->abacusDemo.bookCount].edition == NULL)
+		return 0;
+	return abacusLibrary.book[w->abacusDemo.bookCount].edition[getEdition(w)].chapters;
+}
+
+static unsigned int getNumberLessons(AbacusWidget w) {
+	unsigned int edition = getEdition(w);
+	if (fallback) {
+		return getFallbackLessons(w->abacusDemo.chapterCount);
+	} else {
+		if (abacusLibrary.book[w->abacusDemo.bookCount].edition[edition].chapter == NULL)
+			return 0;
+		return abacusLibrary.book[w->abacusDemo.bookCount].edition[edition].chapter[w->abacusDemo.chapterCount].lessons;
 	}
 }
 
-/* Find the lessons... this way its easy to add more */
-static void
-setLessons(AbacusWidget w)
-{
-	char fname[512], lname[512];
-	int chapter;
-
-	for (chapter = 0; chapter < CHAPTERS; chapter++)
-		w->abacusDemo.lessons[chapter] = 0;
-	for (;;) {
-		for (chapter = 0; chapter < CHAPTERS; chapter++) {
-			w->abacusDemo.lessons[chapter]++;
-			(void) sprintf(fname, "%s%sAbacus%s%d_%d.les",
-				w->abacusDemo.path,
-				FINALDELIM, postfixType(w->abacus.mode),
-				chapter + 1,
-				w->abacusDemo.lessons[chapter]);
-			if ((w->abacusDemo.fp = fopen(fname, "r")) == NULL) {
-				/* Try current directory in case its not installed. */
-				(void) sprintf(lname, "%s%s%sAbacus%s%d_%d.les",
-#ifdef VMS
-					"[",
-#else
-					"",
-#endif
-					"lessons",
-#ifdef VMS
-					"]",
-#else
-					FINALDELIM,
-#endif
-					postfixType(w->abacus.mode),
-					chapter + 1,
-					w->abacusDemo.lessons[chapter]);
-				if ((w->abacusDemo.fp = fopen(lname, "r")) == NULL) {
-					w->abacusDemo.lessons[chapter]--;
-#ifdef DEBUG
-					(void) fprintf(stderr, "No Chapter %d, lesson %d, exiting\n",
-						chapter + 1,
-						w->abacusDemo.lessons[chapter] + 1);
-#endif
-					return;
-				}
-			}
-			(void) fclose(w->abacusDemo.fp);
-		}
+static unsigned int getNumberMoves(AbacusWidget w) {
+	unsigned int edition = getEdition(w);
+	if (fallback) {
+		return getFallbackMoves(edition, w->abacusDemo.chapterCount, w->abacusDemo.lessonCount);
+	} else {
+		if (abacusLibrary.book[w->abacusDemo.bookCount].edition[edition].chapter[w->abacusDemo.chapterCount].lesson == NULL)
+			return 0;
+		return abacusLibrary.book[w->abacusDemo.bookCount].edition[edition].chapter[w->abacusDemo.chapterCount].lesson[w->abacusDemo.lessonCount].moves;
 	}
 }
 
@@ -477,16 +491,9 @@ static void drawLineText(AbacusWidget w, const char* line, int i)
 static void
 drawText(AbacusWidget w, TextType textType)
 {
+	unsigned int edition = getEdition(w);
 	int line;
 
-#if 0
-	static Boolean firstTime = True;
-
-	if (firstTime) {
-		firstTime = False;
-		setLessons(w);
-	}
-#endif
 #if 0
 #if (defined(WINVER) && defined(DEMO_FRAMED) || !defined(WINVER))
 	else
@@ -510,8 +517,18 @@ drawText(AbacusWidget w, TextType textType)
 			0, 0, w->core.width, w->core.height);
 	}
 #endif
+	if (textType == BOOK) {
+		unsigned int realMode = (unsigned int) w->abacus.mode;
+		unsigned int forcedMode = getEdition(w);
+		edition = forcedMode;
+		/* Korean and Japanese are similar and may be OK */
+		if (forcedMode == 0 &&
+				(realMode == RUSSIAN || realMode == DANISH)) {
+			(void) fprintf(stderr, "No Russian edition, expect errors.\n");
+		}
+	}
 	for (line = 0; line < w->abacusDemo.lines; line++) {
-		const char * temp;
+		const char* temp;
 
 		switch (textType) {
 		case INTRO:
@@ -522,13 +539,65 @@ drawText(AbacusWidget w, TextType textType)
 #ifdef DEBUG
 			(void) fprintf(stderr, " INTRO: %s\n", temp);
 #endif
-			w->abacusDemo.chapter = True;
+			w->abacusDemo.chapter = False;
+			w->abacusDemo.book = True;
+			break;
+		case BOOK:
+			if (fallback) {
+				temp = bookTextFallback[w->abacusDemo.bookCount][line];
+#ifdef DEBUG
+				(void) fprintf(stderr, " BOOK: %s\n", temp);
+#endif
+			} else {
+				temp = bookText[line];
+#ifdef DEBUG
+				(void) fprintf(stderr, " BOOK: %s\n", temp);
+#endif
+				if (line == 0) {
+					char buf[512];
+#ifdef HAVE_SNPRINTF
+					(void) snprintf(buf, 512,
+						"%s%s by %s", temp,
+						abacusLibrary.book[w->abacusDemo.bookCount].name,
+						abacusLibrary.book[w->abacusDemo.bookCount].author);
+#else
+					(void) sprintf(buf,
+						"%s%s by %s", temp,
+						abacusLibrary.book[w->abacusDemo.bookCount].name,
+						abacusLibrary.book[w->abacusDemo.bookCount].author);
+#endif
+					drawLineText(w, buf, line);
+					continue;
+				}
+			}
 			break;
 		case CHAPTER:
-			temp = chapterText[w->abacusDemo.chapterCount][line];
+			if (fallback) {
+				temp = chapterTextFallback[w->abacusDemo.bookCount][w->abacusDemo.chapterCount][line];
 #ifdef DEBUG
-			(void) fprintf(stderr, " CHAPTER: %s\n", temp);
+				(void) fprintf(stderr, " CHAPTER: %s\n", temp);
 #endif
+			} else {
+
+				temp = chapterText[line];
+#ifdef DEBUG
+				(void) fprintf(stderr, " CHAPTER: %s\n", temp);
+#endif
+				if (line == 0) {
+					char buf[512];
+#ifdef HAVE_SNPRINTF
+					(void) snprintf(buf, 512,
+						"%s%s", temp,
+						abacusLibrary.book[w->abacusDemo.bookCount].edition[edition].chapter[w->abacusDemo.chapterCount].name);
+#else
+					(void) sprintf(buf,
+						"%s%s", temp,
+						abacusLibrary.book[w->abacusDemo.bookCount].edition[edition].chapter[w->abacusDemo.chapterCount].name);
+#endif
+					drawLineText(w, buf, line);
+					continue;
+				}
+			}
 			break;
 		case CONCL:
 			w->abacusDemo.started = False;
@@ -555,138 +624,96 @@ drawText(AbacusWidget w, TextType textType)
 	}
 }
 
-static void
-startLesson(AbacusWidget w)
-{
-	char fname[512], lname[512], buf[512];
-
-	(void) sprintf(fname, "%s%sAbacus%s%d_%d.les", w->abacusDemo.path,
-		FINALDELIM, postfixType(w->abacus.mode),
-		w->abacusDemo.chapterCount + 1,
-		w->abacusDemo.lessonCount + 1);
-#ifdef DEBUG
-	(void) fprintf(stderr, "Chapter %d, lesson %d, Lesson name = %s\n",
-		w->abacusDemo.chapterCount + 1,
-		w->abacusDemo.lessonCount + 1, fname);
-#endif
-
-	if ((w->abacusDemo.fp = fopen(fname, "r")) == NULL) {
-		/* Try current directory in case its not installed. */
-		(void) sprintf(lname, "%s%s%sAbacus%s%d_%d.les",
-#ifdef VMS
-			"[",
-#else
-			"",
-#endif
-			"lessons",
-#ifdef VMS
-			"]",
-#else
-			FINALDELIM,
-#endif
-			postfixType(w->abacus.mode),
-			w->abacusDemo.chapterCount + 1,
-			w->abacusDemo.lessonCount + 1);
-		if ((w->abacusDemo.fp = fopen(lname, "r")) == NULL) {
-			(void) sprintf(buf,
-				"Could not open lesson script-file: %s; or backup: %s\n",
-				fname, lname);
-			DISPLAY_ERROR(buf);
-		}
-	}
-	w->abacusDemo.lessonLength = 0;
-	while (fscanf(w->abacusDemo.fp, "%d %d %d %d %d\n",
-			&(w->abacusDemo.aux),
-			&(w->abacusDemo.deck), &(w->abacusDemo.rail),
-			&(w->abacusDemo.number), &(w->abacusDemo.lines)) > 0) {
-		int line;
-
-		if (w->abacusDemo.lines > LINES)
-			w->abacusDemo.lines = LINES;
-		for (line = 0; line < w->abacusDemo.lines; line++) {
-			(void) fgets(displayText[line], 64, w->abacusDemo.fp);
-			displayText[line][strlen(displayText[line]) - 1] =
-				'\0';
-		}
-		++w->abacusDemo.lessonLength;
-	}
-	(void) fclose(w->abacusDemo.fp);
-	/* rewind */
-	if ((w->abacusDemo.fp = fopen(fname, "r")) == NULL) {
-		/* Try current directory in case its not installed yet. */
-		(void) sprintf(lname, "%s%s%sAbacus%s%d_%d.les",
-#ifdef VMS
-			"[",
-#else
-			"",
-#endif
-			"lessons",
-#ifdef VMS
-			"]",
-#else
-			FINALDELIM,
-#endif
-			postfixType(w->abacus.mode),
-			w->abacusDemo.chapterCount + 1,
-			w->abacusDemo.lessonCount + 1);
-		if ((w->abacusDemo.fp = fopen(lname, "r")) == NULL) {
-			(void) sprintf(buf,
-				"Could not open lesson script-file: %s; or backup: %s\n",
-				fname, lname);
-			DISPLAY_ERROR(buf);
-		}
-	}
-#ifdef DEBUG
-	(void) fprintf(stderr, "lessonLength = %d\n", w->abacusDemo.lessonLength);
-#endif
-}
-
 static void
 doLesson(AbacusWidget w)
 {
 	int line;
-
-	if (w->abacusDemo.fp == NULL)
-		startLesson(w);
 	/* load first instance */
-	(void) fscanf(w->abacusDemo.fp, "%d %d %d %d %d\n",
-		&(w->abacusDemo.aux),
-		&(w->abacusDemo.deck), &(w->abacusDemo.rail),
-		&(w->abacusDemo.number), &(w->abacusDemo.lines));
+	unsigned int edition = getEdition(w);
+	const char *tmp;
+	unsigned int numberMoves = getNumberMoves(w);
+	if (w->abacusDemo.moveCount < numberMoves) {
+		if (w->abacusDemo.lessonCount >= getNumberLessons(w)) {
 #ifdef DEBUG
-	(void) fprintf(stderr,
-		"aux = %d, deck = %d, rail = %d, number = %d, lines = %d\n",
-		w->abacusDemo.aux,
-		w->abacusDemo.deck, w->abacusDemo.rail,
-		w->abacusDemo.number, w->abacusDemo.lines);
+			(void) fprintf(stderr, "returning doLesson:\n");
 #endif
-	if (w->abacusDemo.lines > LINES)
-		w->abacusDemo.lines = LINES;
-	for (line = 0; line < w->abacusDemo.lines; line++) {
-		(void) fgets(displayText[line], 64, w->abacusDemo.fp);
-		displayText[line][strlen(displayText[line]) - 1] = '\0';
+			w->abacusDemo.chapter = True;
+			return;
+		}
+		if (fallback) {
+			tmp = lessonTextFallback[w->abacusDemo.bookCount][edition][w->abacusDemo.chapterCount][w->abacusDemo.lessonCount][w->abacusDemo.moveCount][0];
+		} else {
+			tmp = abacusLibrary.book[w->abacusDemo.bookCount].edition[edition].chapter[w->abacusDemo.chapterCount].lesson[w->abacusDemo.lessonCount].move[w->abacusDemo.moveCount].code;
+		}
+		(void) sscanf(tmp, "%d %d %d %d %d",
+			&(w->abacusDemo.aux),
+			&(w->abacusDemo.deck), &(w->abacusDemo.rail),
+			&(w->abacusDemo.number), &(w->abacusDemo.lines));
+#ifdef DEBUG
+		(void) fprintf(stderr,
+			"aux = %d, deck = %d, rail = %d, number = %d, lines = %d\n",
+			w->abacusDemo.aux,
+			w->abacusDemo.deck, w->abacusDemo.rail,
+			w->abacusDemo.number, w->abacusDemo.lines);
+#endif
+		if (w->abacusDemo.lines > LINES)
+			w->abacusDemo.lines = LINES;
+		for (line = 0; line < w->abacusDemo.lines; line++) {
+			if (fallback) {
+				tmp = lessonTextFallback[w->abacusDemo.bookCount][edition][w->abacusDemo.chapterCount][w->abacusDemo.lessonCount][w->abacusDemo.moveCount][line + 1];
+			} else {
+				tmp = abacusLibrary.book[w->abacusDemo.bookCount].edition[edition].chapter[w->abacusDemo.chapterCount].lesson[w->abacusDemo.lessonCount].move[w->abacusDemo.moveCount].lineText[line]; /* 64 char */
+			}
+			(void) strncpy(displayText[line], tmp, strlen(tmp) + 1);
+			displayText[line][strlen(displayText[line])] = '\0';
+		}
 	}
 
 	/* A '0' in demo number signifies that only the text is to be
 	 * displayed (i.e. nothing is to be added or subtracted).
 	 */
-
-	if (w->abacusDemo.lessonLength) {
+	if (w->abacusDemo.moveCount <= numberMoves) {
 		setAbacusMove(w, ACTION_MOVE, w->abacusDemo.aux,
 			w->abacusDemo.deck, w->abacusDemo.rail,
 			w->abacusDemo.number);
+		if (w->abacusDemo.highlightAux >= 0 &&
+				w->abacusDemo.highlightRail >= 0) {
+			setAbacusHighlightRail(w, ACTION_UNHIGHLIGHT_RAIL,
+				w->abacusDemo.highlightAux,
+				w->abacusDemo.highlightRail);
+			w->abacusDemo.highlightAux = -1;
+			w->abacusDemo.highlightRail = -1;
+		}
+		if (w->abacusDemo.aux >= 0 &&
+				w->abacusDemo.rail >= 0) {
+			if (w->abacusDemo.number == 0) {
+				w->abacusDemo.highlightAux = -1;
+				w->abacusDemo.highlightRail = -1;
+			} else {
+				w->abacusDemo.highlightAux = w->abacusDemo.aux;
+				w->abacusDemo.highlightRail = w->abacusDemo.rail;
+				setAbacusHighlightRail(w, ACTION_HIGHLIGHT_RAIL,
+					w->abacusDemo.aux, w->abacusDemo.rail);
+			}
+		}
 	}
 #ifdef DEBUG
 	(void) fprintf(stderr, " doLesson:\n");
 #endif
-	if (--w->abacusDemo.lessonLength < 0) {
-		(void) fclose(w->abacusDemo.fp);
+	if (++w->abacusDemo.moveCount > numberMoves) {
+		/*(void) fclose(w->abacusDemo.fp);*/
 		drawText(w, CONCL);
 		Sleep(2000);
+		if (w->abacusDemo.aux >= 0) {
+			setAbacusHighlightRails(w, w->abacusDemo.aux);
+			w->abacusDemo.highlightAux = -1;
+		}
 		drawText(w, QUERY);
 		setAbacusDemo(w, ACTION_CLEAR_NODEMO);
 	} else if (w->abacusDemo.started) {
-		if (w->abacusDemo.chapter) {
+		if (w->abacusDemo.book) {
+			drawText(w, BOOK);
+		} else if (w->abacusDemo.chapter) {
 			drawText(w, CHAPTER);
 		} else {
 			drawText(w, DISPLAY);
@@ -699,43 +726,73 @@ doLesson(AbacusWidget w)
 }
 
 static void
-queryDemo(AbacusWidget w, Boolean chapt, Boolean advance)
+queryDemo(AbacusWidget w, Boolean advance)
 {
 #ifdef DEBUG
-	(void) fprintf(stderr, "queryDemo: chapt %d, advance %d\n",
-		chapt, advance);
+	(void) fprintf(stderr, "queryDemo: advance %d\n",
+		advance);
 #endif
 	w->abacusDemo.started = True;
-	if (chapt) {
+	if (w->abacusDemo.book) {
+		if (advance) {
+			w->abacusDemo.bookCount++;
+			if (w->abacusDemo.bookCount >= getNumberBooks())
+				w->abacusDemo.bookCount = 0;
+			drawText(w, BOOK);
+		} else {
+			/* got a Read */
+			w->abacusDemo.book = False;
+			w->abacusDemo.chapter = True;
+			w->abacusDemo.chapterCount = 0;
+			w->abacusDemo.query = False;
+			drawText(w, CHAPTER);
+		}
+	} else if (w->abacusDemo.chapter) {
 		if (advance) {
 			w->abacusDemo.chapterCount++;
-			if (w->abacusDemo.chapterCount >= CHAPTERS)
+			if (w->abacusDemo.chapterCount >= getNumberChapters(w)) {
 				w->abacusDemo.chapterCount = 0;
-			drawText(w, CHAPTER);
+				w->abacusDemo.bookCount++;
+				if (w->abacusDemo.bookCount >= getNumberBooks()) {
+					w->abacusDemo.bookCount = 0;
+				}
+				w->abacusDemo.book = True;
+				drawText(w, BOOK);
+			} else {
+				drawText(w, CHAPTER);
+			}
 		} else {
 			/* got a Read */
 			w->abacusDemo.chapter = False;
 			w->abacusDemo.lessonCount = 0;
+			w->abacusDemo.moveCount = 0;
 			w->abacusDemo.query = False;
-			startLesson(w);
 			doLesson(w);
 		}
 	} else {
 		if (w->abacusDemo.query) {
 			w->abacusDemo.started = True;
 			w->abacusDemo.query = False;
+			w->abacusDemo.moveCount = 0;
 			if (advance) {
 				w->abacusDemo.lessonCount++;
-				if (w->abacusDemo.lessonCount >= w->abacusDemo.lessons[w->abacusDemo.chapterCount]) {
+				if (w->abacusDemo.lessonCount >= getNumberLessons(w)) {
 					w->abacusDemo.lessonCount = 0;
 					w->abacusDemo.chapterCount++;
-					if (w->abacusDemo.chapterCount >= CHAPTERS)
+					if (w->abacusDemo.chapterCount >= getNumberChapters(w)) {
 						w->abacusDemo.chapterCount = 0;
-					w->abacusDemo.chapter = True;
-					drawText(w, CHAPTER);
+						w->abacusDemo.bookCount++;
+						if (w->abacusDemo.bookCount >= getNumberBooks()) {
+							w->abacusDemo.bookCount = 0;
+						}
+						w->abacusDemo.book = True;
+						drawText(w, BOOK);
+					} else {
+						w->abacusDemo.chapter = True;
+						drawText(w, CHAPTER);
+					}
 				}
 			}
-			startLesson(w);
 			doLesson(w);
 		}
 	}
@@ -747,23 +804,32 @@ jumpDemo(AbacusWidget w)
 	if (!w->abacusDemo.chapter) {
 		w->abacusDemo.chapter = True;
 		w->abacusDemo.started = True;
+		w->abacusDemo.moveCount = 0;
+		w->abacusDemo.lessonCount = 0;
 		w->abacusDemo.chapterCount++;
-		if (w->abacusDemo.chapterCount >= CHAPTERS)
+		if (w->abacusDemo.chapterCount >= getNumberChapters(w)) {
 			w->abacusDemo.chapterCount = 0;
-		setLessons(w);
-		drawText(w, CHAPTER);
+			w->abacusDemo.bookCount++;
+			if (w->abacusDemo.bookCount >= getNumberBooks()) {
+				w->abacusDemo.bookCount = 0;
+			}
+			w->abacusDemo.book = True;
+			drawText(w, BOOK);
+		} else {
+			drawText(w, CHAPTER);
+		}
 	}
 }
 
 static void
-chapterDemo(AbacusWidget w, int chapt)
+chapterDemo(AbacusWidget w, unsigned int chapt)
 {
 	if (w->abacusDemo.started && w->abacusDemo.chapter) {
+		w->abacusDemo.moveCount = 0;
+		w->abacusDemo.lessonCount = 0;
 		w->abacusDemo.chapterCount = chapt;
-		if (w->abacusDemo.chapterCount < 0 ||
-				w->abacusDemo.chapterCount >= CHAPTERS)
+		if (w->abacusDemo.chapterCount >= getNumberChapters(w))
 			w->abacusDemo.chapterCount = 0;
-		setLessons(w);
 		drawText(w, CHAPTER);
 	}
 }
@@ -771,8 +837,8 @@ chapterDemo(AbacusWidget w, int chapt)
 static void
 moreDemo(AbacusWidget w)
 {
-	if (!w->abacusDemo.chapter && w->abacusDemo.started &&
-			!w->abacusDemo.query) {
+	if (!w->abacusDemo.book && !w->abacusDemo.chapter &&
+			w->abacusDemo.started && !w->abacusDemo.query) {
 		doLesson(w);
 	}
 }
@@ -807,11 +873,27 @@ initializeAbacusDemo(
 #endif
 	w->abacusDemo.started = False;
 	w->abacusDemo.query = False;
-	w->abacusDemo.chapter = True;
+	w->abacusDemo.chapter = False;
+	w->abacusDemo.book = True;
+	w->abacusDemo.bookCount = 0;
 	w->abacusDemo.chapterCount = 0;
 	w->abacusDemo.lessonCount = 0;
+	w->abacusDemo.moveCount = 0;
 	w->abacusDemo.lines = LINES;
-	setLessons(w);
+	w->abacusDemo.highlightAux = -1;
+	w->abacusDemo.highlightRail = -1;
+	readParseFile(XML_FILE);
+	if (abacusLibrary.book != NULL) {
+		fallback = False;
+#ifndef WINVER
+	/* always true in Windows version, some runtime issue with libraries */
+	} else {
+		if (displayedError == 0) {
+			(void) fprintf(stderr, "Trouble using XML, using the static fallback single book demo.\n");
+			displayedError = 1;
+		}
+#endif
+	}
 }
 
 #ifndef WINVER
@@ -825,7 +907,6 @@ setValuesAbacusDemo(Widget current, Widget request, Widget renew)
 #endif
 
 	if (w->abacus.mode != c->abacus.mode) {
-		setLessons(w);
 		redraw = True;
 	}
 	if (w->core.background_pixel != c->core.background_pixel ||
@@ -851,11 +932,11 @@ setValuesAbacusDemo(Widget current, Widget request, Widget renew)
 		break;
 	case NEXT_DECK:
 		w->abacus.deck = IGNORE_DECK;
-		queryDemo(w, w->abacusDemo.chapter, True);
+		queryDemo(w, True);
 		break;
 	case REPEAT_DECK:
 		w->abacus.deck = IGNORE_DECK;
-		queryDemo(w, w->abacusDemo.chapter, False);
+		queryDemo(w, False);
 		break;
 	case JUMP_DECK:
 		w->abacus.deck = IGNORE_DECK;
@@ -880,6 +961,11 @@ destroyAbacusDemo(Widget old)
 {
 	AbacusWidget w = (AbacusWidget) old;
 
+	if (abacusLibrary.book != NULL) {
+		freeAbacus(abacusLibrary.book, abacusLibrary.books);
+		abacusLibrary.book = NULL;
+		abacusLibrary.books = 0;
+	}
 	XtReleaseGC(old, w->abacusDemo.foregroundGC);
 	XtReleaseGC(old, w->abacusDemo.inverseGC);
 	XtRemoveCallbacks(old, XtNselectCallback, w->abacus.select);
@@ -916,7 +1002,6 @@ Widget renew, XEvent *event, Region region
 #endif
 )
 {
-
 #ifdef WINVER
 #ifdef DEMO_FRAMED
 	{
@@ -937,7 +1022,9 @@ Widget renew, XEvent *event, Region region
 		0, 0, w->core.width, w->core.height);
 #endif
 	if (w->abacusDemo.started) {
-		if (w->abacusDemo.chapter) {
+		if (w->abacusDemo.book) {
+			drawText(w, BOOK);
+		} else if (w->abacusDemo.chapter) {
 			drawText(w, CHAPTER);
 		} else {
 			drawText(w, DISPLAY);
@@ -945,7 +1032,7 @@ Widget renew, XEvent *event, Region region
 	} else if (w->abacusDemo.query) {
 		drawText(w, QUERY);
 	} else {
-		drawText(w, CHAPTER);
+		drawText(w, BOOK);
 	}
 }
 
@@ -957,6 +1044,11 @@ hideAbacusDemo(AbacusWidget w
 	)
 {
 	setAbacusDemo(w, ACTION_HIDE);
+	if (abacusLibrary.book != NULL) {
+		freeAbacus(abacusLibrary.book, abacusLibrary.books);
+		abacusLibrary.book = NULL;
+		abacusLibrary.books = 0;
+	}
 }
 
 static
@@ -965,7 +1057,7 @@ selectAbacusDemo(AbacusWidget w
 , XEvent *event, char **args, int nArgs
 	)
 {
-	clearAbacusDemo(w);
+	/*clearAbacusDemo(w);*/
 }
 #endif
 
@@ -979,9 +1071,16 @@ clearAbacusDemo(AbacusWidget w)
 	(void) fprintf(stderr, " CLEAR\n");
 #endif
 	w->abacusDemo.started = True;
-	w->abacusDemo.query = True;
-	startLesson(w);
-	doLesson(w);
+	/*w->abacusDemo.query = True;*/
+	w->abacusDemo.moveCount = 0;
+	if (w->abacusDemo.highlightAux >= 0 &&
+			w->abacusDemo.highlightRail >= 0) {
+		setAbacusHighlightRail(w, ACTION_UNHIGHLIGHT_RAIL,
+			w->abacusDemo.highlightAux,
+			w->abacusDemo.highlightRail);
+		w->abacusDemo.highlightAux = -1;
+		w->abacusDemo.highlightRail = -1;
+	}
 }
 
 #ifndef WINVER
@@ -999,10 +1098,13 @@ toggleDemoAbacusDemo(AbacusWidget w
 	if (w->abacus.demo) {
 		setAbacusDemo(w, ACTION_DEMO_DEFAULT);
 		w->abacusDemo.started = False;
+		w->abacusDemo.bookCount = 0;
 		w->abacusDemo.chapterCount = 0;
 		w->abacusDemo.lessonCount = 0;
+		w->abacusDemo.moveCount = 0;
 		w->abacusDemo.query = True;
 		w->abacusDemo.chapter = False;
+		w->abacusDemo.book = False;
 	}
 #else
 	setAbacusDemo(w, ACTION_DEMO);
@@ -1019,7 +1121,7 @@ showNextAbacusDemo(AbacusWidget w
 #endif
 	)
 {
-	queryDemo(w, w->abacusDemo.chapter, True);
+	queryDemo(w, True);
 }
 
 #ifndef WINVER
@@ -1032,7 +1134,7 @@ showRepeatAbacusDemo(AbacusWidget w
 #endif
 	)
 {
-	queryDemo(w, w->abacusDemo.chapter, False);
+	queryDemo(w, False);
 }
 
 #ifndef WINVER
@@ -1063,7 +1165,7 @@ showMoreAbacusDemo(AbacusWidget w
 
 #ifdef WINVER
 void
-showChapterAbacusDemo(AbacusWidget w, int chapt)
+showChapterAbacusDemo(AbacusWidget w, unsigned int chapt)
 {
 	chapterDemo(w, chapt);
 }
