@@ -1,7 +1,7 @@
 /*
  * @(#)AbacusP.h
  *
- * Copyright 1994 - 2016  David A. Bagley, bagleyd AT verizon.net
+ * Copyright 1994 - 2018  David A. Bagley, bagleyd AT verizon.net
  *
  * Abacus demo and neat pointers from
  * Copyright 1991 - 1998  Luis Fernandes, elf AT ee.ryerson.ca
@@ -60,18 +60,22 @@ extern void hideAbacus(AbacusWidget w);
 extern void selectAbacus(AbacusWidget w, const int x, const int y);
 extern void releaseAbacus(AbacusWidget w, const int x, const int y);
 extern void clearAbacus(AbacusWidget w);
+extern void clearDecimalAbacus(AbacusWidget w);
 extern void complementAbacus(AbacusWidget w);
 extern void incrementAbacus(AbacusWidget w);
 extern void decrementAbacus(AbacusWidget w);
 extern void changeFormatAbacus(AbacusWidget w);
 extern void changeMuseumAbacus(AbacusWidget w);
 extern void toggleRomanNumeralsAbacus(AbacusWidget w);
+extern void toggleAncientRomanAbacus(AbacusWidget w);
+extern void toggleLatinAbacus(AbacusWidget w);
 extern void toggleGroupingAbacus(AbacusWidget w);
 extern void toggleNegativeSignAbacus(AbacusWidget w);
 extern void toggleQuartersAbacus(AbacusWidget w);
 extern void toggleQuarterPercentsAbacus(AbacusWidget w);
 extern void toggleTwelfthsAbacus(AbacusWidget w);
 extern void toggleSubdecksAbacus(AbacusWidget w);
+extern void toggleModernRomanAbacus(AbacusWidget w);
 extern void toggleEighthsAbacus(AbacusWidget w);
 extern void toggleAnomalyAbacus(AbacusWidget w);
 extern void toggleWatchAbacus(AbacusWidget w);
@@ -83,7 +87,7 @@ extern void toggleRightToLeftMultAbacus(AbacusWidget w);
 extern void enterAbacus(AbacusWidget w);
 extern void leaveAbacus(AbacusWidget w);
 extern void moveAbacusInput(AbacusWidget w,
-        int x, int y, char letter, int control);
+	int x, int y, char letter, int control);
 extern Boolean checkSubdeck(AbacusWidget w, int position);
 extern void setAbacusString(AbacusWidget w, int reason, char *string);
 
@@ -95,7 +99,7 @@ extern void showNextAbacusDemo(AbacusWidget w);
 extern void showRepeatAbacusDemo(AbacusWidget w);
 extern void showJumpAbacusDemo(AbacusWidget w);
 extern void showMoreAbacusDemo(AbacusWidget w);
-extern void showChapterAbacusDemo(AbacusWidget w, int chapt);
+extern void showChapterAbacusDemo(AbacusWidget w, unsigned int chapt);
 
 #else
 extern Pixel darker(AbacusWidget w, Pixel pixel);
@@ -134,6 +138,9 @@ extern AbacusClassRec abacusClassRec;
 #ifndef DRIPSOUND
 #define DRIPSOUND "drip"
 #endif
+#ifndef XML_FILE
+#define XML_FILE "abacusDemo.xml"
+#endif
 
 #define MAX_DECKS 2
 #define PLACE_SETTING 2
@@ -153,7 +160,8 @@ extern AbacusClassRec abacusClassRec;
 #define MAX_SLICES 10
 #define COUNTRY_SIZE 3
 #define STRING_SIZE 256
-#define MAX_SHADES 12
+#define MAX_BEAD_SHADES 12
+#define MAX_RAIL_SHADES 12
 
 #define NORMAL 1
 #define DOUBLE 2
@@ -212,7 +220,8 @@ typedef struct _AbacusPart {
 	int         subdeck, subbead, subbase;
 	int         shiftPercent, shiftAnomaly, shiftAnomalySq;
 	int         groupSize, anomaly, anomalySq;
-	int         delay, numSlices, mode, submode, colorScheme, railIndex;
+	unsigned int mode;
+	int         delay, numSlices, submode, colorScheme, railIndex;
 	int         leftAuxRails, rightAuxRails;
 	int         aux, deck, rail, number;
 	Position    middleBarHeight, middleBarPositionY, railWidth;
@@ -227,8 +236,9 @@ typedef struct _AbacusPart {
 	Boolean     slot, diamond, sound;
 	char       *digits, *mathBuffer;
 	FILE       *fp;
-	GC          frameGC, borderGC, symbolGC, lineGC, railGC[4], inverseGC;
-	GC          beadShadeGC[MAX_SHADES];
+	GC          frameGC, borderGC, symbolGC, lineGC, inverseGC;
+	GC          beadShadeGC[MAX_BEAD_SHADES];
+	GC          railShadeGC[MAX_BEAD_SHADES];
 	Pixmap      bufferBead[4][2][2];
 	TimeVal     oldTime;
 	Boolean     rightToLeftAdd, rightToLeftMult;
@@ -261,12 +271,12 @@ typedef struct _AbacusPart {
 } AbacusPart;
 
 typedef struct _AbacusDemoPart {
-	int         chapterCount;
-	int         lessons[CHAPTERS], lessonLength, lessonCount;
+	unsigned int bookCount, chapterCount, lessonCount, moveCount;
 	int         aux, deck, rail, number, lines;
+	int         highlightAux, highlightRail;
 	int         fontHeight;
 	FILE       *fp;
-	Boolean     chapter, query, started, framed;
+	Boolean     book, chapter, query, started, framed;
 	GC          foregroundGC;
 #ifdef WINVER
 	char        path[STRING_SIZE];
@@ -285,10 +295,59 @@ typedef struct _AbacusRec {
 	AbacusDemoPart abacusDemo;
 } AbacusRec;
 
+struct moveType {
+	char *code;
+	unsigned int lines;
+	char **lineText;
+};
+
+struct lessonType {
+	char *name;
+	unsigned int moves;
+	struct moveType *move;
+};
+
+struct chapterType {
+	char *name;
+	unsigned int lessons;
+	struct lessonType *lesson;
+};
+
+struct editionType {
+	char *version;
+	unsigned int chapters;
+	struct chapterType *chapter;
+};
+
+struct bookType {
+	char *name;
+	char *author;
+	unsigned int editions;
+	struct editionType *edition;
+};
+
+struct libraryType {
+	unsigned int books;
+	struct bookType *book;
+};
+
+extern unsigned int getFallbackBooks(void);
+extern unsigned int getFallbackEditions(void);
+extern unsigned int getFallbackChapters(void);
+extern unsigned int getFallbackLessons(unsigned int chapter);
+extern unsigned int getFallbackMoves(unsigned int edition, unsigned int chapter, unsigned int lesson);
+
+extern const char *bookTextFallback[1][LINES];
+extern const char *chapterTextFallback[1][5][LINES];
+extern const char *lessonTextFallback[1][4][5][2][41][LINES + 1];
+extern void freeAbacus(struct bookType *bookAbacus, unsigned int books);
+
 extern void setAbacus(AbacusWidget w, int reason);
 extern void setAbacusDemo(AbacusWidget w, int reason);
 extern void setAbacusMove(AbacusWidget w, int reason, int aux,
 	int deck, int rail, int number);
+extern void setAbacusHighlightRail(AbacusWidget w, int reason, int aux, int rail);
+extern void setAbacusHighlightRails(AbacusWidget w, int aux);
 extern void setAbacusText(AbacusWidget w, int reason, char * text, int line);
 extern int char2Int(char character);
 extern char int2Char(int digit);
@@ -310,7 +369,7 @@ extern void convertString(char * buf, char * inbuf,
 	char decimalPoint);
 
 extern int sizeofRoman(int base, Boolean romanNumerals, Boolean ancientRoman);
-extern int string2Roman(char *buf, char *inbuf, int base,
+extern int string2Roman(char *buf, char *inbuf, int base, Boolean decimalFraction,
 	int pieces, int number, int subnumber, int subbase,
 	char decimalPoint, Boolean ancientRoman, Boolean latin);
 extern void string2Group(char *buf, char *inbuf, int groupSize,
@@ -326,9 +385,10 @@ extern int convertBaseToBottom(int base);
 extern int rootInt(int i, int n);
 
 extern void clearRails(AbacusWidget w);
+extern void clearDecimalRails(AbacusWidget w);
 extern void addBackAnomaly(char * buf, int anomaly, int shift, int base);
 extern void zeroFractionalPart(char * buf);
-extern void convertStringToAbacus(AbacusWidget w, char *string, int aux);
+extern void convertStringToAbacus(AbacusWidget w, const char *string, int aux);
 extern double convertToDecimal(int base, char *inputString);
 extern void convertFromInteger(char *outputString, int base, int x);
 extern void convertFromDouble(char *outputString, int base, double x);
@@ -339,4 +399,9 @@ extern void drawTeachText(const char* line, int i);
 extern void clearAuxRails(AbacusWidget w, int aux);
 #endif
 
+extern void highlightRail(AbacusWidget w, int aux, int rail,
+	Boolean highlight);
+extern void highlightRails(AbacusWidget w, int aux);
+extern void readParseFile(const char * fileName);
+
 #endif /* _AbacusP_h */
